@@ -21,10 +21,12 @@
 
 #include <hildon-mime.h>
 #include <sharing-http.h>
+#include <sharing-tag.h>
 #include <libxml/parser.h>
 
 #define TWITPIC_API_KEY                 "1f9ce28260728df0a40cafe3506a9407"
 #define MOBYPICTURE_API_KEY             "L5RL7tAoAAsgZqKP"
+#define YFROG_API_KEY                   "45ENOPRS1f5a98c9f89b523a8465d2a3ad0602a3"
 
 static gchar *
 parse_server_response                   (const gchar       *response,
@@ -47,6 +49,7 @@ parse_server_response                   (const gchar       *response,
       break;
     case SERVICE_MOBYPICTURE:
     case SERVICE_TWITGOO:
+    case SERVICE_YFROG:
       rootid = "rsp";
       urlid  = "mediaurl";
       break;
@@ -204,7 +207,7 @@ twitmulti_account_setup                 (SharingAccount *account,
   gtk_container_add (vbox, label);
   gtk_container_add (vbox, button);
 
-  g_signal_connect (button, "clicked", G_CALLBACK (register_account_clicked), account);
+  g_signal_connect (button, "clicked", G_CALLBACK (register_account_clicked), NULL);
 
   gtk_widget_show_all (d);
   response = gtk_dialog_run (GTK_DIALOG (d));
@@ -267,6 +270,8 @@ get_twitter_pic_service                 (SharingEntry *entry)
         service = SERVICE_IMGLY;
       else if (g_str_equal (servicename, "posterous"))
         service = SERVICE_POSTEROUS;
+      else if (g_str_equal (servicename, "yfrog"))
+        service = SERVICE_YFROG;
     }
 
   return service;
@@ -324,6 +329,7 @@ twitmulti_share_file                    (SharingTransfer *transfer,
       else
         {
           const gchar *posturl;
+          const gchar *verify_url = TWITTER_VERIFY_CREDENTIALS_JSON;
           gchar *hdr, *title;
           SharingHTTP *http = sharing_http_new ();
           SharingHTTPRunResponse httpret;
@@ -342,6 +348,26 @@ twitmulti_share_file                    (SharingTransfer *transfer,
           if (title)
             g_strstrip (title);
 
+          /* We support Mobypicture tags */
+          if (service == SERVICE_MOBYPICTURE)
+            {
+              const GSList *taglist = sharing_entry_media_get_tags (media);
+              if (taglist != NULL)
+                {
+                  GString *tags = g_string_sized_new (100);
+                  while (taglist != NULL)
+                    {
+                      SharingTag *tag = (SharingTag *) taglist->data;
+                      g_string_append (tags, sharing_tag_get_word (tag));
+                      taglist = taglist->next;
+                      if (taglist != NULL)
+                        g_string_append_c (tags, ',');
+                    }
+                  sharing_http_add_req_multipart_data (http, "tags", tags->str, -1, "text/plain");
+                  g_string_free (tags, TRUE);
+                }
+            }
+
           sharing_http_set_progress_callback (http, upload_progress_cb, &data);
           sharing_http_add_req_multipart_file (http, "media", path, mime);
           if (title)
@@ -366,15 +392,20 @@ twitmulti_share_file                    (SharingTransfer *transfer,
             case SERVICE_POSTEROUS:
               posturl = "https://posterous.com/api2/upload.xml";
               break;
+            case SERVICE_YFROG:
+              sharing_http_add_req_multipart_data (http, "key", YFROG_API_KEY, -1, "text/plain");
+              posturl = "https://yfrog.com/api/xauth_upload";
+              verify_url = TWITTER_VERIFY_CREDENTIALS_XML;
+              break;
             default:
               g_return_val_if_reached (SHARING_SEND_ERROR_UNKNOWN);
             }
 
-          hdr = twitter_get_verify_credentials_header (account);
+          hdr = twitter_get_verify_credentials_header (account, verify_url);
           sharing_http_add_req_header_line (http, hdr);
           g_free (hdr);
 
-          sharing_http_add_req_header_line (http, "X-Auth-Service-Provider: " TWITTER_VERIFY_CREDENTIALS_URL);
+          sharing_http_add_req_header (http, "X-Auth-Service-Provider", verify_url);
           httpret = sharing_http_run (http, posturl);
 
           switch (httpret)
